"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const types_1 = require("./types");
const useFetchArgs_1 = __importDefault(require("./useFetchArgs"));
const use_ssr_1 = __importDefault(require("use-ssr"));
const makeRouteAndOptions_1 = __importDefault(require("./makeRouteAndOptions"));
const utils_1 = require("./utils");
function useFetch(...args) {
    const { customOptions, requestInit, defaults } = useFetchArgs_1.default(...args);
    const { url, onMount, onUpdate, path, interceptors, timeout, retries } = customOptions;
    const { isServer } = use_ssr_1.default();
    const controller = react_1.useRef();
    const res = react_1.useRef();
    const data = react_1.useRef(defaults.data);
    const timedout = react_1.useRef(false);
    const attempts = react_1.useRef(retries);
    const [loading, setLoading] = react_1.useState(defaults.loading);
    const [error, setError] = react_1.useState();
    const makeFetch = react_1.useCallback((method) => {
        const doFetch = (routeOrBody, body) => __awaiter(this, void 0, void 0, function* () {
            if (isServer)
                return; // for now, we don't do anything on the server
            const theController = controller.current = new AbortController();
            setLoading(true);
            setError(undefined);
            let { route, options } = yield makeRouteAndOptions_1.default(requestInit, method, theController, routeOrBody, body, interceptors.request);
            const timer = timeout > 0 && setTimeout(() => {
                timedout.current = true;
                theController.abort();
            }, timeout);
            let theData;
            try {
                res.current = yield fetch(`${url}${path}${route}`, options);
                try {
                    theData = yield res.current.json();
                }
                catch (err) {
                    theData = (yield res.current.text()); // FIXME: should not be `any` type
                }
            }
            catch (err) {
                if (attempts.current > 0)
                    return doFetch(routeOrBody, body);
                if (attempts.current < 1 && timedout.current)
                    setError({ name: 'AbortError', message: 'Timeout Error' });
                if (err.name !== 'AbortError')
                    setError(err);
            }
            finally {
                if (attempts.current > 0)
                    attempts.current -= 1;
                timedout.current = false;
                if (timer)
                    clearTimeout(timer);
                data.current = (defaults.data && utils_1.isEmpty(theData)) ? defaults.data : theData;
                controller.current = undefined;
                setLoading(false);
            }
            return data.current;
        });
        return doFetch;
    }, [url, requestInit, isServer]);
    const post = makeFetch(types_1.HTTPMethod.POST);
    const del = makeFetch(types_1.HTTPMethod.DELETE);
    const request = {
        get: makeFetch(types_1.HTTPMethod.GET),
        post,
        patch: makeFetch(types_1.HTTPMethod.PATCH),
        put: makeFetch(types_1.HTTPMethod.PUT),
        del,
        delete: del,
        abort: () => controller.current && controller.current.abort(),
        query: (query, variables) => post({ query, variables }),
        mutate: (mutation, variables) => post({ mutation, variables }),
        loading: loading,
        error,
        data: data.current,
    };
    const responseObj = Object.assign({ data: data.current }, res.current);
    const response = interceptors.response ? interceptors.response(responseObj) : responseObj;
    const executeRequest = react_1.useCallback(() => {
        const methodName = requestInit.method || types_1.HTTPMethod.GET;
        const methodLower = methodName.toLowerCase();
        if (methodName !== types_1.HTTPMethod.GET) {
            const req = request[methodLower];
            req(requestInit.body);
        }
        else {
            const req = request[methodLower];
            req();
        }
    }, [requestInit.body, requestInit.method]);
    const mounted = react_1.useRef(false);
    // handling onUpdate
    react_1.useEffect(() => {
        if (onUpdate.length === 0 || !mounted.current)
            return;
        executeRequest();
    }, [...onUpdate, executeRequest]);
    // handling onMount
    react_1.useEffect(() => {
        if (mounted.current)
            return;
        mounted.current = true;
        if (!onMount)
            return;
        executeRequest();
    }, [onMount, executeRequest]);
    return Object.assign([request, response, loading, error], Object.assign({ request, response: response }, request));
}
exports.useFetch = useFetch;
exports.default = useFetch;
//# sourceMappingURL=useFetch.js.map